#include <stdio.h>
#include <wiringPi.h>
#include <wiringPiI2C.h>

//  LCD表示器 AQM1248A 制御関数
//  LCDWriteRS0(int fd, int data)
//  LCDWriteRS1(int fd, int data)
LCDWriteRS0(int fd, int data) {
    wiringPiI2CWriteReg8(fd, 0x00, data);
}

LCDWriteRS1(int fd, int data) {
    wiringPiI2CWriteReg8(fd, 0x40, data);
}

//   LCD表示器 AQM1248A 初期化
//   LCDInit(int fd)
LCDInit(int fd) {
    int i;

    delay(40);
    LCDWriteRS0(fd, 0x38); delayMicroseconds(27);	// Function Set IS=0
    LCDWriteRS0(fd, 0x39); delayMicroseconds(27);	// Function Set IS=1
    LCDWriteRS0(fd, 0x14); delayMicroseconds(27);	// BS=0, Int. OSC=0b100
    LCDWriteRS0(fd, 0x73); delayMicroseconds(27);	// Contrast Set
    LCDWriteRS0(fd, 0x56); delayMicroseconds(27);	// Power/ICON/Contrast control
    LCDWriteRS0(fd, 0x6c); delay(200);			// Follower control
    LCDWriteRS0(fd, 0x38); delayMicroseconds(27);	// Function set IS=0
    LCDWriteRS0(fd, 0x01); delay(2);			// Clear display
    LCDWriteRS0(fd, 0x0d); delayMicroseconds(27);	// DisplayOn CursorOff
}

//   LCD表示器 AQM1248A 文字を表示する関数
//   LCDputc(int fd, int ch)
//   ch には表示させたい文字コードを入れる
LCDputchar(int fd, char ch) {
    LCDWriteRS1(fd, ch);	// Write data to RAM
    delayMicroseconds(27);
}

//   LCD表示器 AQM1248A 画面をクリアする関数
//   cls(int fd)
cls(int fd){
    LCDWriteRS0(fd, 0x01);	// Clear display
    delay(2);
}

//   LCD表示器 AQM1248A 文字出力位置指定関数
//   locate(int fd, int x, int y)
//   xは水平位置 0から15までの整数。
//   yは垂直位置 0 か 1 の整数。
//
//   ■場所指定方法
//    -------------AQM1602XA----------------
//    | ロロロロロ▲ロロロロロロロロロロロロ|
//    | ロロ●ロロロロロロロロロロロロロロロ|
//    ---------------------------------------
//    例１：上図で▲を指定するとき
//	　locate(fd, 5,0);
//    例２：上図で●を指定するとき
//	  locate(fd, 2,1);
locate(int fd, int x,int y) {
     int data;

     data = 0x80;
     if (y==0) data |=  x;
	else   data |=  x | 0x40;
     LCDWriteRS0(fd, data);		// Set DDRAM Address
     delayMicroseconds(27);
}

//   cursor(int fd, int cursorVal)
//   点滅カーソルを表示 /　非表示する関数
//   cursor(fd , 1)  カーソルを表示 (0以外)
//   cursor(fd , 0)  カーソルを非表示
cursor(int fd, int cursorVal) {
    if (cursorVal)
	LCDWriteRS0(fd, 0x0d);		// cursor on
        else LCDWriteRS0(fd, 0x0c);	// cursor off
    delayMicroseconds(27);
}

//   slcd(int fd, char buf)
//   文字列表示ﾌﾟﾛｸﾞﾗﾑ sprintf(buf,"[文字列]\0")をやった後の文字列をLCDに表示
//   最後に'\0'がないと、無限ループになるので注意。
slcd(int fd, char buf[]) {
   while(*buf != '\0')
   {
	LCDputchar(fd, *buf);
        buf++;
   }
}

//
// メイン関数
//
main() {
    int fd;
    char str[20];

    wiringPiSetup();			// WiringPi初期化
    fd = wiringPiI2CSetup(0x3e);	// AQM1602XA
    LCDInit(fd);			// LCD初期化
    cursor(fd, 0);			// カーソル非表示

   locate(fd, 0, 0); sprintf(str,"CQ Publishing   \0"); slcd(fd, str);
   locate(fd, 0, 1); sprintf(str,"TransistorGijuts\0"); slcd(fd, str);
}

