#include <stdio.h>
#include <wiringPi.h>
#include <wiringPiI2C.h>

//  LCD表示器 AQM1248A 制御関数
//  LCDWriteRS0(int fd, int data)
//  LCDWriteRS1(int fd, int data)
LCDWriteRS0(int fd, int data) {
    wiringPiI2CWriteReg8(fd, 0x00, data);
}

LCDWriteRS1(int fd, int data) {
    wiringPiI2CWriteReg8(fd, 0x40, data);
}

//   LCD表示器 AQM1248A 初期化
//   LCDInit(int fd)
LCDInit(int fd) {
    int i;

    delay(40);
    LCDWriteRS0(fd, 0x38); delayMicroseconds(27);	// Function Set IS=0
    LCDWriteRS0(fd, 0x39); delayMicroseconds(27);	// Function Set IS=1
    LCDWriteRS0(fd, 0x14); delayMicroseconds(27);	// BS=0, Int. OSC=0b100
    LCDWriteRS0(fd, 0x73); delayMicroseconds(27);	// Contrast Set
    LCDWriteRS0(fd, 0x56); delayMicroseconds(27);	// Power/ICON/Contrast control
    LCDWriteRS0(fd, 0x6c); delay(200);			// Follower control
    LCDWriteRS0(fd, 0x38); delayMicroseconds(27);	// Function set IS=0
    LCDWriteRS0(fd, 0x01); delay(2);			// Clear display
    LCDWriteRS0(fd, 0x0d); delayMicroseconds(27);	// DisplayOn CursorOff
}

//   LCD表示器 AQM1248A 文字を表示する関数
//   LCDputc(int fd, int ch)
//   ch には表示させたい文字コードを入れる
LCDputchar(int fd, char ch) {
    LCDWriteRS1(fd, ch);	// Write data to RAM
    delayMicroseconds(27);
}

//   LCD表示器 AQM1248A 画面をクリアする関数
//   cls(int fd)
cls(int fd){
    LCDWriteRS0(fd, 0x01);	// Clear display
    delay(2);
}

//   LCD表示器 AQM1248A 文字出力位置指定関数
//   locate(int fd, int x, int y)
//   xは水平位置 0から15までの整数。
//   yは垂直位置 0 か 1 の整数。
//
//   ■場所指定方法
//    -------------AQM1602XA----------------
//    | ロロロロロ▲ロロロロロロロロロロロロ|
//    | ロロ●ロロロロロロロロロロロロロロロ|
//    ---------------------------------------
//    例１：上図で▲を指定するとき
//	　locate(fd, 5,0);
//    例２：上図で●を指定するとき
//	  locate(fd, 2,1);
locate(int fd, int x,int y) {
     int data;

     data = 0x80;
     if (y==0) data |=  x;
	else   data |=  x | 0x40;
     LCDWriteRS0(fd, data);		// Set DDRAM Address
     delayMicroseconds(27);
}

//   cursor(int fd, int cursorVal)
//   点滅カーソルを表示 /　非表示する関数
//   cursor(fd , 1)  カーソルを表示 (0以外)
//   cursor(fd , 0)  カーソルを非表示
cursor(int fd, int cursorVal) {
    if (cursorVal)
	LCDWriteRS0(fd, 0x0d);		// cursor on
        else LCDWriteRS0(fd, 0x0c);	// cursor off
    delayMicroseconds(27);
}

//   slcd(int fd, char buf)
//   文字列表示ﾌﾟﾛｸﾞﾗﾑ sprintf(buf,"[文字列]\0")をやった後の文字列をLCDに表示
//   最後に'\0'がないと、無限ループになるので注意。
slcd(int fd, char buf[]) {
   while(*buf != '\0')
   {
	LCDputchar(fd, *buf);
        buf++;
   }
}

// LPS25Hの初期化
// LPS25HInit(int fd)
// fdはファイルハンドル
LPS25HInit(int fd) {
    wiringPiI2CWriteReg8(fd, 0x20, 0x90); // 1秒おきにデータ取得
}

// LPS25Hから気圧取得
// float LPS25HPressure(int fd)
// fdはファイルハンドル
float LPS25HPressure(int fd) {
    int PRESS_OUT;
    int press;

    PRESS_OUT  = wiringPiI2CReadReg8(fd, 0x28);         // PRESS_OUT_XL
    PRESS_OUT |= wiringPiI2CReadReg8(fd, 0x29) << 8;    // PRESS_OUT_H
    PRESS_OUT |= wiringPiI2CReadReg8(fd, 0x2a) << 16;   // PRESS_OUT_L
    press = PRESS_OUT / 4096.0;         // 気圧に変換

    return(press);
}

// LPS25Hから温度取得
// float LPS25HTemperature(int fd)
// fdはファイルハンドル
float LPS25HTemperature(int fd) {
    int TEMP_OUT;
    float temp;

    TEMP_OUT   = wiringPiI2CReadReg8(fd, 0x2b);         // TEMP_OUT_L
    TEMP_OUT  |= wiringPiI2CReadReg8(fd, 0x2c) << 8;    // TEMP_OUT_H
    // 16bit符号付き整数 -> システムの符号つき変数へ拡張
    if ((TEMP_OUT & 0x8000) != 0) TEMP_OUT |= (~0x7fff);

    temp = 42.5 + TEMP_OUT / 480.0;     // 温度に変換

    return(temp);
}

//  HDC1000を初期化する関数
//  HDC1000Init(fd)
// fdはファイルハンドル
HDC1000Init(int fd) {
    unsigned char buf[4];

    buf[0]=0x02;        // Configuration Resistor
    buf[1]=0x00;        // MODE 0
    buf[2]=0x00;        // Reserved
    write(fd, buf, 3);

    return(fd);
}

// HDC1000から温度を読み取る関数
// float HDC1000Temperature(int fd)
// fdはファイルハンドル
// 返り値は摂氏温度[℃]
float HDC1000Temperature(int fd) {
    unsigned char buf[4];
    float temperature;

    buf[0]=0x00;                // 0x00にアクセス
    write(fd, buf, 1);
    delay(7);                   // ウェイト 6.35ms -> 7ms
    read(fd, buf, 2);           // データ読み出し

    // 物理量へ変換
    temperature = (buf[0] << 8 | buf[1]) / 65536.0 * 165.0 - 40;

    return(temperature);
}

// HDC1000から湿度を読み取る関数
// float HDC1000Humidity(int fd)
// fdはファイルハンドル
// 返り値は相対湿度[%]
float HDC1000Humidity(int fd) {
    unsigned char buf[4];
    float humidity;

    buf[0]=0x01;                // 0x01にアクセス
    write(fd, buf, 1);
    delay(7);                   // ウェイト 6.5ms -> 7ms
    read(fd, buf, 2);           // データ読み出し

    // 物理量へ変換
    humidity = (buf[0] << 8 | buf[1]) / 65536.0 * 100.0;

    return(humidity);
}

//
// メイン関数
//
main() {
    int fdAQM1602;
    int fdLPS25H;
    int fdHDC1000;
    float temperature, humidity, pressure;
    char str[20];


    wiringPiSetup();			// WiringPi初期化
    fdAQM1602 = wiringPiI2CSetup(0x3e);	// AQM1602XAファイルハンドル取得
    LCDInit(fdAQM1602);			// LCD初期化
    cursor(fdAQM1602, 0);		// カーソル非表示

    fdLPS25H = wiringPiI2CSetup(0x5c);	// LPS25Hファイルハンドル取得
    LPS25HInit(fdLPS25H);		// LPS25Hの初期化

    fdHDC1000 = wiringPiI2CSetup(0x40);	// HDC1000ファイルハンドル取得
    HDC1000Init(fdHDC1000);		// HDC1000の初期化

   for(;;) {

        temperature = HDC1000Temperature(fdHDC1000);
        humidity    = HDC1000Humidity(fdHDC1000);
	pressure    = LPS25HPressure(fdLPS25H);

        printf("%2.1f deg   %3.0f %%   %4.1f hPa\n",
			temperature, humidity, pressure);

   	locate(fdAQM1602, 0, 0);
   	sprintf(str,"%2.1f deg   %3.0f %%\0",temperature, humidity);
   	slcd(fdAQM1602, str);

   	locate(fdAQM1602, 0, 1);
   	sprintf(str,"%4.1f hPa\0", pressure);
   	slcd(fdAQM1602, str);

	delay(1000);
    }
}

